import logging
from typing import List, Optional

from ouro._resource import SyncAPIResource

log: logging.Logger = logging.getLogger(__name__)


__all__ = ["Assets"]


class Assets(SyncAPIResource):
    def __init__(self, client):
        super().__init__(client)

    def search(
        self,
        query: str,
        **kwargs,
    ) -> List[dict]:
        """
        Search for assets
        """

        request = self.client.get(
            f"/users/search",
            params={
                "query": query,
                **kwargs,
            },
        )
        request.raise_for_status()
        response = request.json()
        if response.get("error", None):
            raise Exception(response["error"])
        return response.get("data", [])



import logging
from typing import List, Optional

from ouro._resource import SyncAPIResource
from ouro.models import Post

from .content import Content, Editor

log: logging.Logger = logging.getLogger(__name__)


__all__ = ["Posts"]


class Posts(SyncAPIResource):
    def __init__(self, client):
        super().__init__(client)

    @staticmethod
    def Editor(**kwargs) -> Editor:
        return Editor(**kwargs)

    @staticmethod
    def Content(**kwargs) -> Content:
        return Content(**kwargs)

    def create(
        self,
        content: Content,
        name: str,
        description: Optional[str] = None,
        visibility: Optional[str] = None,
        monetization: Optional[str] = None,
        price: Optional[float] = None,
        **kwargs,
    ) -> Post:
        """
        Create a new Post
        """

        post = {
            "name": name,
            "description": description,
            "visibility": visibility,
            "monetization": monetization,
            "price": price,
            **kwargs,
        }
        # Filter out None values
        post = {k: v for k, v in post.items() if v is not None}

        request = self.client.post(
            "/posts/create",
            json={
                "post": post,
                "content": content.to_dict(),
            },
        )
        request.raise_for_status()
        response = request.json()
        if response["error"]:
            raise Exception(response["error"])

        return Post(**response["data"])

    def retrieve(self, id: str):
        """
        Retrieve a Post by its id
        """
        request = self.client.get(
            f"/posts/{id}",
        )
        request.raise_for_status()
        response = request.json()
        if response["error"]:
            raise Exception(response["error"])

        return Post(**response["data"])

    def update(
        self,
        id: str,
        content: Optional[Content] = None,
        name: Optional[str] = None,
        description: Optional[str] = None,
        visibility: Optional[str] = None,
        monetization: Optional[str] = None,
        price: Optional[float] = None,
        **kwargs,
    ) -> Post:
        """
        Update a Post by its id
        """

        post = {
            "name": name,
            "description": description,
            "visibility": visibility,
            "monetization": monetization,
            "price": price,
            **kwargs,
        }
        # Filter out None values
        post = {k: v for k, v in post.items() if v is not None}

        request = self.client.put(
            f"/posts/{id}",
            json={
                "post": post,
                "content": content.to_dict() if content is not None else None,
            },
        )
        request.raise_for_status()
        response = request.json()
        if response["error"]:
            raise Exception(response["error"])
        return Post(**response["data"])



import asyncio
import logging
from typing import Callable, List, Optional

from ouro._resource import SyncAPIResource
from ouro.models import Conversation
from ouro.realtime.websocket import OuroWebSocket

from .content import Content

log: logging.Logger = logging.getLogger(__name__)


__all__ = ["Conversations", "Messages"]


class Messages(SyncAPIResource):
    def create(self, conversation_id: str, **kwargs):
        json = kwargs.get("json")
        text = kwargs.get("text")
        user_id = kwargs.get("user_id")
        message = {
            "json": json,
            "text": text,
            "user_id": user_id,
            **kwargs,
        }

        message = {k: v for k, v in message.items() if v is not None}
        request = self.client.post(
            f"/conversations/{conversation_id}/messages/create",
            json={"message": message},
        )
        request.raise_for_status()
        response = request.json()
        if response["error"]:
            raise Exception(response["error"])
        return response["data"]

    # def retrieve(self, id: str):
    #     request = self.client.get(f"/messages/{id}")
    #     request.raise_for_status()
    #     response = request.json()
    #     if response["error"]:
    #         raise Exception(response["error"])
    #     return response["data"]

    # def update(self, id: str, content: Optional[Content] = None, **kwargs):
    #     message = {**kwargs}
    #     message = {k: v for k, v in message.items() if v is not None}
    #     request = self.client.put(
    #         f"/messages/{id}",
    #         json={"message": message, "content": content.to_dict() if content else None},
    #     )
    #     request.raise_for_status()
    #     response = request.json()
    #     if response["error"]:
    #         raise Exception(response["error"])
    #     return response["data"]

    # def delete(self, id: str):
    #     request = self.client.delete(f"/messages/{id}")
    #     request.raise_for_status()
    #     response = request.json()
    #     if response["error"]:
    #         raise Exception(response["error"])
    #     return response["data"]

    def list(self, conversation_id: str, **kwargs):
        request = self.client.get(
            f"/conversations/{conversation_id}/messages", params=kwargs
        )
        request.raise_for_status()
        response = request.json()
        if response["error"]:
            raise Exception(response["error"])
        return response["data"]


class ConversationMessages:
    def __init__(self, conversation: "Conversation"):
        self.conversation = conversation
        self._ouro = conversation._ouro

    def create(self, **kwargs):
        return Messages(self._ouro).create(self.conversation.id, **kwargs)

    def retrieve(self, message_id: str):
        return Messages(self._ouro).retrieve(message_id)

    def update(self, message_id: str, content: Optional[Content] = None, **kwargs):
        return Messages(self._ouro).update(message_id, content, **kwargs)

    def delete(self, message_id: str):
        return Messages(self._ouro).delete(message_id)

    def list(self, **kwargs):
        return Messages(self._ouro).list(self.conversation.id, **kwargs)


class Conversations(SyncAPIResource):
    # def __init__(self, *args, **kwargs):
    #     super().__init__(*args, **kwargs)
    #     self.messages = Messages(*args, **kwargs)

    def retrieve(self, conversation_id: str):
        """
        Retrieve a conversation by id.
        """

        request = self.client.get(f"/conversations/{conversation_id}")
        request.raise_for_status()
        response = request.json()
        if response["error"]:
            raise Exception(response["error"])
        conversation = response["data"]
        return Conversation(**conversation, _ouro=self.ouro)

    def list(self, **kwargs):
        """
        List all conversations.
        """
        request = self.client.get("/conversations", params=kwargs)
        request.raise_for_status()
        response = request.json()
        if response["error"]:
            raise Exception(response["error"])
        return [
            Conversation(**conversation, _ouro=self.ouro)
            for conversation in response["data"]
        ]

    def update(self, conversation_id: str, **kwargs):
        """
        Update a conversation.
        """
        request = self.client.put(
            f"/conversations/{conversation_id}", json={"conversation": kwargs}
        )
        request.raise_for_status()
        response = request.json()
        if response["error"]:
            raise Exception(response["error"])

        return Conversation(**response["data"], _ouro=self.ouro)

    async def subscribe(self):
        """
        Subscribe to conversation changes. New messages will be sent to the websocket connection.
        """
        # Check to make sure the websocket connection is active
        if not self.websocket.is_connected:
            await self.websocket.connect(self.ouro.websocket_url)
        request = self.client.get("/conversations/subscribe")
        request.raise_for_status()
        response = request.json()
        if response.get("error"):
            raise Exception(response["error"])
        log.info("Subscribed to conversations")
        return response["data"]

    async def unsubscribe(self):
        """
        Unsubscribe from conversation changes.
        """
        if self.websocket.is_connected:
            await self.websocket.disconnect()



from __future__ import annotations

# from ._models import BaseModel
from ouro.__version__ import __title__, __version__
from ouro._client import Ouro
from ouro.config import Config
from typing_extensions import override

# from ._response import APIResponse as APIResponse, AsyncAPIResponse as AsyncAPIResponse
from ._constants import DEFAULT_CONNECTION_LIMITS, DEFAULT_MAX_RETRIES, DEFAULT_TIMEOUT
from ._exceptions import (
    APIConnectionError,
    APIError,
    APIResponseValidationError,
    APIStatusError,
    APITimeoutError,
    AuthenticationError,
    BadRequestError,
    ConflictError,
    InternalServerError,
    NotFoundError,
    OuroError,
    PermissionDeniedError,
    RateLimitError,
    UnprocessableEntityError,
)
from ._logs import setup_logging
from ._types import NOT_GIVEN, NoneType, ProxiesTypes, Transport

# from ._base_client import DefaultHttpxClient, DefaultAsyncHttpxClient
# from ._utils._logs import setup_logging as _setup_logging

__all__ = [
    # "types",
    "__version__",
    "__title__",
    # "NoneType",
    # "Transport",
    # "ProxiesTypes",
    # "NotGiven",
    # "NOT_GIVEN",
    # "OuroError",
    # "APIError",
    # "APIStatusError",
    # "APITimeoutError",
    # "APIConnectionError",
    # "APIResponseValidationError",
    # "BadRequestError",
    # "AuthenticationError",
    # "PermissionDeniedError",
    # "NotFoundError",
    # "ConflictError",
    # "UnprocessableEntityError",
    # "RateLimitError",
    # "InternalServerError",
    # "Timeout",
    # "RequestOptions",
    # "Client",
    "Ouro",
    # "DEFAULT_TIMEOUT",
    # "DEFAULT_MAX_RETRIES",
    # "DEFAULT_CONNECTION_LIMITS",
]

setup_logging()

# Update the __module__ attribute for exported symbols so that
# error messages point to this module instead of the module
# it was originally defined in, e.g.
# ouro._exceptions.NotFoundError -> ouro.NotFoundError
__locals = locals()
for __name in __all__:
    if not __name.startswith("__"):
        try:
            __locals[__name].__module__ = "ouro"
        except (TypeError, AttributeError):
            # Some of our exported symbols are builtins which we can't set attributes for.
            pass



from datetime import datetime
from typing import TYPE_CHECKING, Any, List, Literal, Optional, Union
from uuid import UUID

from pydantic import BaseModel, ConfigDict, Field

if TYPE_CHECKING:
    from ouro.resources.conversations import ConversationMessages

    from ouro import Ouro

__all__ = [
    "Asset",
    "PostContent",
    "Post",
    "Conversation",
    "File",
    "FileData",
    "Dataset",
]


class Asset(BaseModel):
    id: UUID
    user_id: UUID
    org_id: UUID | None
    name: str
    visibility: str
    asset_type: str
    created_at: datetime
    last_updated: datetime
    description: Optional[str]
    metadata: Optional[dict]
    monetization: Optional[str]
    price: Optional[float]
    product_id: Optional[str]
    price_id: Optional[str]
    preview: Optional[dict]
    cost_accounting: Optional[str]
    cost_unit: Optional[str]
    unit_cost: Optional[float]


class PostContent(BaseModel):
    text: str
    data: dict = Field(
        alias="json",
    )


class Post(Asset):
    content: Optional[PostContent] = None
    # preview: Optional[PostContent]
    comments: Optional[int] = Field(default=0)
    views: Optional[int] = Field(default=0)


class ConversationMetadata(BaseModel):
    members: List[UUID]
    summary: Optional[str] = None


class Conversation(Asset):
    id: UUID
    name: str
    description: Optional[str] = None
    summary: Optional[str] = None
    metadata: ConversationMetadata
    _messages: Optional["ConversationMessages"] = None
    _ouro: Optional["Ouro"] = None

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self._ouro = kwargs.get("_ouro")

    @property
    def messages(self):
        if self._messages is None:
            from ouro.resources.conversations import ConversationMessages

            self._messages = ConversationMessages(self)
        return self._messages


class FileData(BaseModel):
    signedUrl: Optional[str] = None
    publicUrl: Optional[str] = None


class FileMetadata(BaseModel):
    id: UUID
    name: str
    path: str
    size: int
    type: str
    bucket: Literal["public-files", "files"]
    fullPath: str


class File(Asset):
    metadata: FileMetadata
    data: Optional[FileData] = None


class DatasetMetadata(BaseModel):
    table_name: str
    columns: List[str]


class Dataset(Asset):
    # metadata: Union[DatasetMetadata, Optional[FileMetadata]]
    preview: Optional[List[dict]] = None



import asyncio
import logging
from typing import Any, Callable

import socketio

log = logging.getLogger(__name__)


class OuroWebSocket:
    def __init__(self, ouro):
        self.ouro = ouro
        self.sio = socketio.AsyncClient()
        self.websocket_connected = asyncio.Event()
        self.connection_lock = asyncio.Lock()

    @property
    def is_connected(self):
        return self.websocket_connected.is_set()

    async def connect(self, url: str):
        async with self.connection_lock:
            if not self.is_connected:
                try:
                    await self.sio.connect(
                        url, auth={"access_token": self.ouro.access_token}
                    )
                    self.websocket_connected.set()

                    # Add some default event listeners
                    @self.sio.on("connect")
                    def connect_handler():
                        log.info("Connected to websocket")

                    @self.sio.on("disconnect")
                    def disconnect_handler():
                        log.info("Disconnected from websocket")
                        asyncio.create_task(self.handle_disconnect())

                except Exception as e:
                    log.error(f"Failed to connect to websocket: {e}")
                    raise

    async def handle_disconnect(self):
        self.websocket_connected.clear()
        # Implement reconnection logic here if needed

    async def disconnect(self):
        if self.is_connected:
            await self.sio.disconnect()
            self.websocket_connected.clear()

    def on(self, event: str, handler: Callable):
        self.sio.on(event, handler)

    async def emit(self, event: str, data: Any):
        if not self.is_connected:
            raise RuntimeError("Cannot emit event: WebSocket is not connected")
        await self.sio.emit(event, data)
